//===========================================================================
// GoTools Core - SINTEF Geometry Tools Core library, version 2.0.1
//
// Copyright (C) 2000-2007, 2010 SINTEF ICT, Applied Mathematics, Norway.
//
// This program is free software; you can redistribute it and/or          
// modify it under the terms of the GNU General Public License            
// as published by the Free Software Foundation version 2 of the License. 
//
// This program is distributed in the hope that it will be useful,        
// but WITHOUT ANY WARRANTY; without even the implied warranty of         
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          
// GNU General Public License for more details.                           
//
// You should have received a copy of the GNU General Public License      
// along with this program; if not, write to the Free Software            
// Foundation, Inc.,                                                      
// 59 Temple Place - Suite 330,                                           
// Boston, MA  02111-1307, USA.                                           
//
// Contact information: E-mail: tor.dokken@sintef.no                      
// SINTEF ICT, Department of Applied Mathematics,                         
// P.O. Box 124 Blindern,                                                 
// 0314 Oslo, Norway.                                                     
//
// Other licenses are also available for this software, notably licenses
// for:
// - Building commercial software.                                        
// - Building software whose source code you wish to keep private.        
//===========================================================================

#include "GoTools/geometry/SplineSurface.h"

using namespace std;

namespace Go {


//==========================================================================
void SplineSurface::makeBernsteinKnotsU()
//==========================================================================
{
    // @@ WARNING: Comparing floating point numbers for equality.

    vector<double> new_knots;
    SplineSurface the_surface = *this;
    int k = order_u(); // Order of spline surface in u-direction

    // Loop through u-knot vector
    vector<double>::const_iterator iter = the_surface.basis_u().begin();
    while (iter != the_surface.basis_u().end()) {
	// Finds how many equal knots there are
	int counter = 1;
	while (iter+counter != the_surface.basis_u().end()
	       && *iter == *(iter+counter))
	    ++counter;

	ALWAYS_ERROR_IF(counter > k, "More equal knots than order.");

	// Fill up knots
	new_knots.insert(new_knots.end(), k - counter, *iter);

	iter += counter;
    }

    the_surface.insertKnot_u(new_knots);
    *this = the_surface;
    return;
}

//==========================================================================
void SplineSurface::makeBernsteinKnotsV()
//==========================================================================
{
    // @@ WARNING: Comparing floating point numbers for equality.

    vector<double> new_knots;
    SplineSurface the_surface = *this;
    int k = order_v(); // Order of spline surface in v-direction

    // Loop through v-knot vector
    vector<double>::const_iterator iter = the_surface.basis_v().begin();
    while (iter != the_surface.basis_v().end()) {
	// Finds how many equal knots there are
	int counter = 1;
	while (iter+counter != the_surface.basis_v().end()
	       && *iter == *(iter + counter))
	    ++counter;

	ALWAYS_ERROR_IF(counter > k, "More equal knots than order.");


	// Fill up knots
	new_knots.insert(new_knots.end(), k - counter, *iter);

	iter += counter;
    }

    the_surface.insertKnot_v(new_knots);
    *this = the_surface;
    return;
}


//==========================================================================
int SplineSurface::numberOfPatches_u() const
//==========================================================================
{
    // @@ WARNING: Comparing floating point numbers for equality.

    // Loop through u-knot vector
    vector<double>::const_iterator iter = basis_u().begin()+order_u()-1;
    int num = 0;
    while (iter != basis_u().begin()+numCoefs_u()) {
	int counter = 1;
	// Finds how many equal knots there are
	while (*iter == *(iter+counter)
	       && iter+counter != basis_u().begin()+numCoefs_u())
	    ++counter;
	++num;
	iter += counter;
    }

    return num;
}


//==========================================================================
int SplineSurface::numberOfPatches_v() const
//==========================================================================
{
    // @@ WARNING: Comparing floating point numbers for equality.

    // Loop through u-knot vector
    vector<double>::const_iterator iter = basis_v().begin()+order_v()-1;
    int num = 0;
    while (iter != basis_v().begin()+numCoefs_v()) {
	int counter = 1;
	// Finds how many equal knots there are
	while (*iter == *(iter+counter)
	       && iter+counter != basis_v().begin()+numCoefs_v())
	    ++counter;
	++num;
	iter += counter;
    }

    return num;
}


} // namespace Go;
